from datazen import DESCRIPTION as DESCRIPTION, VERSION as VERSION
from datazen.app import add_app_args as add_app_args, entry as entry
from typing import List

def main(argv: List[str] = ...) -> int: ...
