from typing import Any

PKG_NAME: str
VERSION: str
DESCRIPTION: str
DEFAULT_TYPE: str
DEFAULT_MANIFEST: Any
DEFAULT_DIR: Any
CACHE_SUFFIX: str
GLOBAL_KEY: str

def to_private(input_str: str) -> str: ...

ROOT_NAMESPACE: Any
