from datazen.load import DEFAULT_LOADS as DEFAULT_LOADS, LoadedFiles as LoadedFiles, load_dir as load_dir
from typing import List

def load(directories: List[str], loads: LoadedFiles = ...) -> dict: ...
