from datazen import PKG_NAME as PKG_NAME, VERSION as VERSION
from datazen.parsing import get_hash as get_hash
from typing import List, Tuple

BARRIER: str

def get_comment_data(file_data: str, dynamic: bool = ...) -> List[Tuple[str, str]]: ...
def apply_barriers(comment_lines: List[str], char: str) -> None: ...
def resolve_encapsulation(comment_lines: List[str], file_ext: str) -> str: ...
def build_fingerprint(file_data: str, file_ext: str, char: str = ..., dynamic: bool = ...) -> str: ...
