# =====================================
# generator=datazen
# version=1.7.8
# hash=4c8620a30a13b93abde0ea0c3c0e156a
# =====================================

"""
vmklib - Useful defaults and other package metadata.
"""

PKG_NAME = "vmklib"
VERSION = "0.5.0"
DESCRIPTION = "Simplify project workflows by standardizing use of GNU Make."
