from datazen.compile import write_dir as write_dir
from datazen.load import load_dir_only as load_dir_only
from typing import Any

class TaskDataCache:
    data: Any
    cache_dir: Any
    def __init__(self, cache_dir: str): ...
    def load(self, load_dir: str) -> None: ...
    def save(self, out_type: str = ...) -> None: ...
    def clean(self, purge_data: bool = ...) -> None: ...
