import logging
from datazen.targets import MatchData as MatchData, match_target as match_target, parse_targets as parse_targets, resolve_target_data as resolve_target_data
from typing import Any, List, Optional

class TargetResolver:
    literals: Any
    patterns: Any
    logger: Any
    def __init__(self, logger: logging.Logger = ...) -> None: ...
    def clear(self) -> None: ...
    def get_target(self, group: str, name: str) -> Optional[dict]: ...
    def register_group(self, name: str, targets: List[dict]) -> None: ...
