from datazen import PKG_NAME as PKG_NAME
from typing import Any, Iterator, List, Tuple

FMT_OPEN: str
FMT_CLOSE: str
EXCLUDES: Any

def get_file_name(full_path: str) -> str: ...
def get_file_ext(full_path: str) -> str: ...
def get_path_list(root_abs_path: str, current_abs_path: str) -> List[str]: ...
def format_resolve_delims(value: str, fmt_data: dict, delim: str = ..., delim_replace: str = ...) -> str: ...
def unflatten_dict(data: dict, delim: str = ...) -> dict: ...
def advance_dict_by_path(path_list: List[str], data: dict) -> dict: ...
def get_package_data(relative_path: str) -> str: ...
def get_package_dir(relative_path: str) -> str: ...
def resolve_dir(data: str, rel_base: str = ...) -> str: ...
def walk_with_excludes(path: str, excludes: List[str] = ...) -> Iterator[Tuple[str, List[str], List[str]]]: ...
